
/**************************************************************************
 *
 *  $Id: mbgddmsg.h 1.11 2017/07/05 14:31:23 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and function prototypes to deal with messages
 *    generated by kernel mode drivers. Some type of messages are
 *    always generated, others are only generated ibn DEBUG builds
 *    and are defined void in release builds.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgddmsg.h $
 *  Revision 1.11  2017/07/05 14:31:23  martin
 *  Added _mbg_kdd_msg...() macros.
 *  Code cleanup.
 *  Updated function prototypes.
 *  Revision 1.10  2012/10/02 18:33:21Z  martin
 *  Support for *BSD.
 *  Also enable debug msgs if MBG_DEBUG is defined.
 *  Revision 1.8  2009/04/22 09:54:55  martin
 *  Include mbg_tgt.h also if building without DEBUG.
 *  Revision 1.7  2009/03/19 15:22:54  martin
 *  Cleaned up debug levels.
 *  Revision 1.6  2008/12/05 13:31:47  martin
 *  Use do {} while (0) syntax to avoid potential syntax problems.
 *  Added _mbgddmsg_7().
 *  Revision 1.5  2006/06/19 15:26:19  martin
 *  Fixed compiler warnings if DEBUG or DBG not defined.
 *  Revision 1.4  2002/06/12 12:25:54  martin
 *  Bug fix: check for target MBG_TGT_WIN32 instead of MBG_TGT_W32.
 *  Revision 1.3  2002/02/19 14:50:48Z  MARTIN
 *  Added support for Win32.
 *  Revision 1.2  2002/02/19 09:28:00  MARTIN
 *  Use new header mbg_tgt.h to check the target environment.
 *  Revision 1.1  2001/03/02 13:51:23  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#ifndef _MBGDDMSG_H
#define _MBGDDMSG_H


/* Other headers to be included */

#include <mbg_tgt.h>

#if defined( MBG_TGT_NETWARE )
  #include <conio.h>
#elif defined( MBG_TGT_OS2 )
  #include <iprintf.h>
#elif defined( MBG_TGT_WIN32 )
  #include <ntddk.h>
#elif defined( MBG_TGT_LINUX )
  #include <linux/module.h>
  #include <linux/version.h>
#elif defined( MBG_TGT_BSD )
  // nothing to include
#else  // MBG_TGT_QNX, MBG_TGT_DOS, ...
  //##+++++++++++++
  #include <stdio.h>
#endif


#ifdef _MBGDDMSG
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )
  #pragma pack( 1 )      // set byte alignment
  #define _USING_BYTE_ALIGNMENT
#endif

#ifdef __cplusplus
extern "C" {
#endif


#if defined( MBG_TGT_NETWARE )
  #define _printf ConsolePrintf
  #define _hd
  #define _tl "\n"
#elif defined( MBG_TGT_OS2 )
  #define _printf iprintf
  #define _hd
  #define _tl "\n"
#elif defined( MBG_TGT_WIN32 )
  #define _printf mbg_kdd_msg
  #define _hd
  #define _tl
  #define USE_MBG_KDD_MSG  1
#elif defined( MBG_TGT_LINUX )
  #if ( LINUX_VERSION_CODE < KERNEL_VERSION( 2, 6, 8 ) )
    // vprintk not supported by the kernel
    #define _printf printk
    #define _hd KERN_INFO
    #define _tl "\n"
  #else
    #define _printf mbg_kdd_msg
    #define _hd KERN_NOTICE
    #define _tl "\n"
    #define USE_MBG_KDD_MSG  1
  #endif
#elif defined( MBG_TGT_BSD )
  #define _printf printf
  #define _hd
  #define _tl "\n"
#else  // MBG_TGT_QNX, MBG_TGT_DOS, ...
  #define _printf printf
  #define _hd
  #define _tl "\n"
#endif


#if !defined( USE_MBG_KDD_MSG )
  #define USE_MBG_KDD_MSG  0
#endif


#define _mbg_kdd_msg_0( _fmt ) \
do {                           \
  _printf( _hd _fmt _tl );     \
} while ( 0 )

#define _mbg_kdd_msg_1( _fmt, _p1 ) \
do {                                \
  _printf( _hd _fmt _tl, (_p1) );   \
} while ( 0 )

#define _mbg_kdd_msg_2( _fmt, _p1, _p2 ) \
do {                                     \
  _printf( _hd _fmt _tl, (_p1), (_p2) ); \
} while ( 0 )

#define _mbg_kdd_msg_3( _fmt, _p1, _p2, _p3 )   \
do {                                            \
  _printf( _hd _fmt _tl, (_p1), (_p2), (_p3) ); \
} while ( 0 )

#define _mbg_kdd_msg_4( _fmt, _p1, _p2, _p3, _p4 )     \
do {                                                   \
  _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4) ); \
} while ( 0 )

#define _mbg_kdd_msg_5( _fmt, _p1, _p2, _p3, _p4, _p5 )       \
do {                                                          \
  _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4), (_p5) ); \
} while ( 0 )

#define _mbg_kdd_msg_6( _fmt, _p1, _p2, _p3, _p4, _p5, _p6 )         \
do {                                                                 \
  _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4), (_p5), (_p6) ); \
} while ( 0 )

#define _mbg_kdd_msg_7( _fmt, _p1, _p2, _p3, _p4, _p5, _p6, _p7 )           \
do {                                                                        \
  _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4), (_p5), (_p6), (_p7) ); \
} while ( 0 )



#if defined( DEBUG ) || ( defined( DBG ) && DBG ) || defined( MBG_DEBUG )

enum
{
  MBG_DBG_ERR,
  MBG_DBG_WARN,
  MBG_DBG_INFO,
  MBG_DBG_DETAIL,
  MBG_DBG_INIT_DEV,
  MBG_DEBUG_SEM,
  MBG_DBG_IRQ,
  N_MBG_DBG_LVL
};


#define _chk_lvl( _lvl )  ( (_lvl) < debug )

#define _mbgddmsg_0( _lvl, _fmt )       \
do {                                    \
  if ( _chk_lvl( _lvl ) )               \
    { _mbg_kdd_msg_0( _fmt ); } \
} while ( 0 )

#define _mbgddmsg_1( _lvl, _fmt, _p1 )         \
do {                                           \
  if ( _chk_lvl( _lvl ) )                      \
    { _mbg_kdd_msg_1( _fmt, (_p1) ); } \
} while ( 0 )

#define _mbgddmsg_2( _lvl, _fmt, _p1, _p2 )           \
do {                                                  \
  if ( _chk_lvl( _lvl ) )                             \
    { _mbg_kdd_msg_2( _fmt, (_p1), (_p2) ); } \
} while ( 0 )

#define _mbgddmsg_3( _lvl, _fmt, _p1, _p2, _p3 )             \
do {                                                         \
  if ( _chk_lvl( _lvl ) )                                    \
    { _mbg_kdd_msg_3( _fmt, (_p1), (_p2), (_p3) ); } \
} while ( 0 )

#define _mbgddmsg_4( _lvl, _fmt, _p1, _p2, _p3, _p4 )               \
do {                                                                \
  if ( _chk_lvl( _lvl ) )                                           \
    { _mbg_kdd_msg_4( _fmt, (_p1), (_p2), (_p3), (_p4) ); } \
} while ( 0 )

#define _mbgddmsg_5( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5 )                 \
do {                                                                       \
  if ( _chk_lvl( _lvl ) )                                                  \
    { _mbg_kdd_msg_5( _fmt, (_p1), (_p2), (_p3), (_p4), (_p5) ); } \
} while ( 0 )

#define _mbgddmsg_6( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5, _p6 )                   \
do {                                                                              \
  if ( _chk_lvl( _lvl ) )                                                         \
    { _mbg_kdd_msg_6( _fmt, (_p1), (_p2), (_p3), (_p4), (_p5), (_p6) ); } \
} while ( 0 )

#define _mbgddmsg_7( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5, _p6, _p7 )                     \
do {                                                                                     \
  if ( _chk_lvl( _lvl ) )                                                                \
    { _mbg_kdd_msg_7( _fmt, (_p1), (_p2), (_p3), (_p4), (_p5), (_p6), (_p7) ); } \
} while ( 0 )

#else

  #define _mbgddmsg_0( _lvl, _fmt )                                     _nop_macro_fnc()
  #define _mbgddmsg_1( _lvl, _fmt, _p1 )                                _nop_macro_fnc()
  #define _mbgddmsg_2( _lvl, _fmt, _p1, _p2 )                           _nop_macro_fnc()
  #define _mbgddmsg_3( _lvl, _fmt, _p1, _p2, _p3 )                      _nop_macro_fnc()
  #define _mbgddmsg_4( _lvl, _fmt, _p1, _p2, _p3, _p4 )                 _nop_macro_fnc()
  #define _mbgddmsg_5( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5 )            _nop_macro_fnc()
  #define _mbgddmsg_6( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5, _p6 )       _nop_macro_fnc()
  #define _mbgddmsg_7( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5, _p6, _p7 )  _nop_macro_fnc()

#endif


/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 __attribute__( ( format( printf, 1, 2 ) ) ) void mbg_kdd_msg( const char *fmt, ... ) ;

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USING_BYTE_ALIGNMENT )
  #pragma pack()      // set default alignment
  #undef _USING_BYTE_ALIGNMENT
#endif

/* End of header body */


#undef _ext
#undef _DO_INIT

#endif  /* _MBGDDMSG_H */
