
/**************************************************************************
 *
 *  $Id: mbgutil.h 1.18 2017/07/05 16:44:35 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions used with mbgutil.c.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgutil.h $
 *  Revision 1.18  2017/07/05 16:44:35  martin
 *  New version code 0x0400, compatibility version still 0x0110.
 *  Include timeutil.h.
 *  windows.h is now included in mbg_tgt.h.
 *  Updated function prototypes.
 *  Revision 1.17  2012/10/15 10:08:32  martin
 *  Include stdlib.h.
 *  Cleaned up handling of pragma pack().
 *  Revision 1.16  2009/08/14 10:11:53Z  daniel
 *  New version code 306, compatibility version still 110.
 *  Revision 1.15  2009/06/09 08:57:47Z  daniel
 *  Rev No. 305
 *  Revision 1.14  2009/03/19 09:06:00Z  daniel
 *  New version code 304, compatibility version still 110.
 *  Revision 1.13  2009/01/12 09:35:41Z  daniel
 *  New version code 303, compatibility version still 110.
 *  Updated function prototypes.
 *  Revision 1.12  2008/01/17 10:15:26Z  daniel
 *  New version code 302, compatibility version still 110.
 *  Revision 1.11  2007/10/16 10:01:17Z  daniel
 *  New version code 301, compatibility version still 110.
 *  Revision 1.9  2007/03/21 16:48:31Z  martin
 *  New version code 219, compatibility version still 110.
 *  Revision 1.8  2006/08/09 13:18:18Z  martin
 *  New version code 218, compatibility version still 110.
 *  Revision 1.7  2006/06/08 10:48:52Z  martin
 *  New version code 217, compatibility version still 110.
 *  Added macro _mbg_strncpy().
 *  Revision 1.6  2006/05/10 10:56:54Z  martin
 *  Updated function prototypes.
 *  Revision 1.5  2006/05/02 13:24:49Z  martin
 *  New version code 216, compatibility version still 110.
 *  Revision 1.4  2006/01/11 12:24:05Z  martin
 *  New version code 215, compatibility version still 110.
 *  Revision 1.3  2005/12/15 10:01:51Z  martin
 *  New version 214, compatibility version still 110.
 *  Revision 1.2  2005/02/18 15:13:42Z  martin
 *  Updated function prototypes.
 *  Revision 1.1  2005/02/18 10:39:49Z  martin
 *  Initial revision
 *
 **************************************************************************/

#ifndef _MBGUTIL_H
#define _MBGUTIL_H


/* Other headers to be included */

#include <mbg_tgt.h>
#include <timeutil.h>
#include <use_pack.h>
#include <pcpsdefs.h>
#include <mbggeo.h>
#include <pci_asic.h>

#include <stdlib.h>


#define MBGUTIL_VERSION         0x0400

#define MBGUTIL_COMPAT_VERSION  0x0110


#if defined( MBG_TGT_WIN32 )


#elif defined( MBG_TGT_LINUX )


#elif defined( MBG_TGT_OS2 )


#elif defined( MBG_TGT_DOS )

  #if !defined( MBG_USE_DOS_TSR )
  #endif

#else

#endif

#ifdef _MBGUTIL
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )
  #pragma pack( 1 )      // set byte alignment
  #define _USING_BYTE_ALIGNMENT
#endif


#ifdef __cplusplus
extern "C" {
#endif


// The macro below can be used to simplify the API call if
// a string variable is used rather than a char *.
#define _mbg_strncpy( _s, _src ) \
  mbg_strncpy( _s, sizeof( _s ), _src )


/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 /**
 * @brief Get the DLL/shared library's version code
 *
 * @return The version code at which the library was built
 */
 _MBG_API_ATTR int _MBG_API mbgutil_get_version( void ) ;

 /**
 * @brief Check the DLL/shared library's compatibility
 *
 * @param[in]  header_version  Version code defined in the header file when the application is built
 *
 * @return ::MBG_SUCCESS if compatible, else ::MBG_ERR_LIB_NOT_COMPATIBLE
 */
 _MBG_API_ATTR int _MBG_API mbgutil_check_version( int header_version ) ;

 /**
 * @brief A portable, safe implementation of snprintf()
 *
 * The output string buffer is in any case properly terminated by 0.
 * For a detailed description see ::vsnprintf_safe
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   fmt      Format string according to subsequent parameters
 * @param[in]   ...      Variable argument list according to the format string
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 *
 * @see ::vsnprintf_safe
 * @see ::snprintf_safe
 */
 _MBG_API_ATTR int __attribute__( ( format( printf, 3, 4 ) ) ) _MBG_API mbg_snprintf( char *s, size_t max_len, const char * fmt, ... ) ;

 /**
 * @brief A portable, safe implementation of strncpy()
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the destination string buffer
 * @param[in]   max_len  Size of the destination string buffer
 * @param[in]   src      Pointer to the source string buffer
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_strncpy( char *s, size_t max_len, const char *src ) ;

 /**
 * @brief Write a character multiple times to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   c        The character to write to the output buffer
 * @param[in]   n        The number of characters to write to the output buffer
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_strchar( char *s, size_t max_len, char c, size_t n ) ;

 /**
 * @brief Write a short date string "dd.mm." to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   mday     Day-of-month number, 1..31
 * @param[in]   month    Month number, 1..12
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_date_short( char *s, int max_len, int mday, int month ) ;

 /**
 * @brief Write a date string "dd.mm.yyyy" to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   mday     Day-of-month number, 1..31
 * @param[in]   month    Month number, 1..12
 * @param[in]   year     Year number
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_date( char *s, int max_len, int mday, int month, int year ) ;

 /**
 * @brief Write a short time string "hh:mm" to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   hour     Hours number, 0..23
 * @param[in]   min      Minutes number, 0..59
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_time_short( char *s, int max_len, int hour, int min ) ;

 /**
 * @brief Write a time string "hh:mm:ss" to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   hour     Hours number, 0..23
 * @param[in]   min      Minutes number, 0..59
 * @param[in]   sec      Seconds number, 0..59, or 60 in case of leap second
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_time( char *s, int max_len, int hour, int min, int sec ) ;

 /**
 * @brief Write a long time string "hh:mm:ss.cc" to a string buffer
 *
 * Include 100ths of seconds.
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   hour     Hours number, 0..23
 * @param[in]   min      Minutes number, 0..59
 * @param[in]   sec      Seconds number, 0..59, or 60 in case of leap second
 * @param[in]   sec100   Hundreths of seconds, 0..99
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_time_long( char *s, int max_len, int hour, int min, int sec, int sec100 ) ;

 /**
 * @brief Write a full date and time string to a string buffer
 *
 * The number of space characters between date and time
 * is determined by the global variable ::mbg_date_time_dist.
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::TM_GPS structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_tm_gps_date_time( char *s, int max_len, const TM_GPS *pt ) ;

 /**
 * @brief Write the short date given as ::PCPS_TIME structure to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_date_short( char *s, int max_len, const PCPS_TIME *pt ) ;

 /**
 * @brief Write the date given as ::PCPS_TIME structure to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_date( char *s, int max_len, const PCPS_TIME *pt ) ;

 /**
 * @brief Write the short time given as ::PCPS_TIME structure to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_time_short( char *s, int max_len, const PCPS_TIME *pt ) ;

 /**
 * @brief Write the time given as ::PCPS_TIME structure to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_time( char *s, int max_len, const PCPS_TIME *pt ) ;

 /**
 * @brief Write the time including sec100ths given as ::PCPS_TIME structure to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_time_long( char *s, int max_len, const PCPS_TIME *pt ) ;

 /**
 * @brief Write date and time given as ::PCPS_TIME structure to a string buffer
 *
 * The number of space characters between date and time
 * is determined by the global variable ::mbg_date_time_dist.
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_TIME structure providing date and time
 * @param[in]   tz_str   Optional time zone string to be appended, currently not used, may be NULL
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_date_time( char *s, int max_len, const PCPS_TIME *pt, const char *tz_str ) ;

 /**
 * @brief Write date derived from seconds-since-epoch to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   sec      Number of seconds since the epoch to be converted to date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_date( char *s, int max_len, uint32_t sec ) ;

 /**
 * @brief Write time derived from seconds-since-epoch to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   sec      Number of seconds since the epoch to be converted to date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_time( char *s, int max_len, uint32_t sec ) ;

 /**
 * @brief Write UTC date and time given as ::PCPS_HR_TIME structure to a string buffer
 *
 * The number of space characters between date and time
 * is determined by the global variable ::mbg_date_time_dist.
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_date_time_utc( char *s, int max_len, const PCPS_HR_TIME *pt ) ;

 /**
 * @brief Write local date and time given as ::PCPS_HR_TIME structure to a string buffer
 *
 * The number of space characters between date and time
 * is determined by the global variable ::mbg_date_time_dist.
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_date_time_loc( char *s, int max_len, const PCPS_HR_TIME *pt ) ;

 /**
 * @brief Print binary ::PCPS_FRAC_32 fractions in decimal to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   frac     Binary fractions of a second in ::PCPS_FRAC_32 format
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_time_frac( char *s, int max_len, uint32_t frac ) ;

 /**
 * @brief Print the UTC offset from a ::PCPS_HR_TIME structure to a string buffer
 *
 * The output format is sign - hours - minutes, e.g. "+01:45h".
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date, and UTC offset
 * @param[in]   info     An informational text to be prepended
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_time_offs( char *s, int max_len, const PCPS_HR_TIME *pt, const char *info ) ;

 /**
 * @brief Write a high resolution UTC time stamp including fractions to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_tstamp_utc( char *s, int max_len, const PCPS_HR_TIME *pt ) ;

 /**
 * @brief Write a high resolution local time stamp including fractions to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_tstamp_loc( char *s, int max_len, const PCPS_HR_TIME *pt ) ;

 /**
 * @brief Write a raw high resolution time stamp to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_tstamp_raw( char *s, int max_len, const PCPS_TIME_STAMP *pt ) ;

 /**
 * @brief Write a raw high resolution time stamp plus converted local time to a string buffer
 *
 * The output string also has the time status code appended as hex number.
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure providing date and time
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pcps_hr_time_raw( char *s, int max_len, const PCPS_HR_TIME *pt ) ;

 /**
 * @brief Write time capture / user capture time stamp to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pt       Pointer to a ::PCPS_HR_TIME structure containing a user capture event
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_ucap( char *s, int max_len, const PCPS_HR_TIME *pt ) ;

 /**
 * @brief Write a geographic coordinate in degrees - minutes - seconds to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   pdms     Pointer to a ::DMS structure containing the coordinate
 * @param[in]   prec     Number of digits of the geographic seconds after the decimal separator
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pos_dms( char *s, int max_len, const DMS *pdms, int prec ) ;

 /**
 * @brief Write a position's altitude parameter to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   alt      The altitude parameter, in meters
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pos_alt( char *s, int max_len, double alt ) ;

 /**
 * @brief Write geographic coordinates  to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s        Pointer to the output buffer
 * @param[in]   max_len  Size of the output buffer
 * @param[in]   ppos     Pointer to a ::POS structure containing the coordinates
 * @param[in]   prec     Number of digits of the geographic seconds after the decimal separator
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_pos( char *s, int max_len, const POS *ppos, int prec ) ;

 /**
 * @brief Write device info to a string buffer
 *
 * The output string buffer is in any case properly terminated by 0.
 *
 * @param[out]  s             Pointer to the output buffer
 * @param[in]   max_len       Size of the output buffer
 * @param[in]   short_name    Short device name, e.g. in ::MBG_DEV_NAME format
 * @param[in]   fw_rev_num    The firmware revision number
 * @param[in]   asic_ver_num  The ASIC version number
 *
 * @return the number of characters written to the output buffer, except the terminating 0
 */
 _MBG_API_ATTR int _MBG_API mbg_str_dev_name( char *s, int max_len, const char *short_name, uint16_t fw_rev_num, PCI_ASIC_VERSION asic_ver_num ) ;


/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USING_BYTE_ALIGNMENT )
  #pragma pack()      // set default alignment
  #undef _USING_BYTE_ALIGNMENT
#endif

/* End of header body */


#undef _ext
#undef _DO_INIT

#endif  /* _MBGUTIL_H */

