
/**************************************************************************
 *
 *  $Id: pcpsmktm.c 1.5 2017/07/05 08:05:11 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Function to convert PCPS_TIME to Unix time (seconds since 1970)
 *
 * -----------------------------------------------------------------------
 *  $Log: pcpsmktm.c $
 *  Revision 1.5  2017/07/05 08:05:11  martin
 *  Let pcps_mktime() return a 'time_t' instead of 'long', and made
 *  the PCPS_TIME pointer parameter 'const'.
 *  Added doxygen comments.
 *  Revision 1.4  2006/12/14 15:27:49  martin
 *  Include time.h.
 *  Revision 1.3  2006/08/22 09:10:03  martin
 *  Renamed function totalsec() to mbg_mktime() and moved it
 *  to a separate file mbgmktm.c.
 *  Revision 1.2  2001/08/14 11:58:08  MARTIN
 *  Included sys/time.h for time_t definition.
 *  Revision 1.1  2001/02/02 15:30:09  MARTIN
 *
 **************************************************************************/

#define _PCPSMKTM
 #include <pcpsmktm.h>
#undef _PCPSMKTM

#include <mbgmktm.h>



/*HDR*/
/**
 * @brief Compute a linear time_t value from ::PCPS_TIME
 *
 * This function works like the standard mktime() function but does not
 * account for a timezone setting configured for the standard C library.
 * Instead, since ::PCPS_TIME contains local time with specified UTC offset
 * the UTC offset is removed to yield UTC time.
 *
 * @param[in] tp     A timestamp in ::PCPS_TIME format
 *
 * @return seconds since 1970-01-01 (Unix time_t format) or ((time_t) -1) if range overflow
 */
time_t pcps_mktime( const PCPS_TIME *tp )
{
  time_t secs;
  int year = tp->year;

  // Expand 2 digit year number (year of the century) to 4 digits
  // including the century.
  if ( year < 70 )
    year += 100;

  // Plausibility checks are made by mbg_mktime().
  secs = mbg_mktime( year, tp->month - 1, tp->mday - 1,
                     tp->hour, tp->min, tp->sec );

  // Convert to UTC if result is valid.
  if ( secs != ( (time_t) -1 ) )
    secs -= tp->offs_utc * 3600UL;

  return secs;

}  // pcps_mktime



